package xyz.twovb.sgm.games.impl.capturethebrick;

/*
 * Created by 2vb - 5/7/2024
 */

import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.minimessage.MiniMessage;
import net.kyori.adventure.text.minimessage.tag.resolver.Placeholder;
import net.kyori.adventure.text.minimessage.tag.resolver.TagResolver;
import net.kyori.adventure.text.serializer.legacy.LegacyComponentSerializer;
import net.kyori.adventure.text.serializer.plain.PlainTextComponentSerializer;
import net.megavex.scoreboardlibrary.api.sidebar.Sidebar;
import net.megavex.scoreboardlibrary.api.sidebar.component.ComponentSidebarLayout;
import net.megavex.scoreboardlibrary.api.sidebar.component.SidebarComponent;
import net.megavex.scoreboardlibrary.api.sidebar.component.animation.CollectionSidebarAnimation;
import net.megavex.scoreboardlibrary.api.sidebar.component.animation.SidebarAnimation;
import net.megavex.scoreboardlibrary.api.team.ScoreboardTeam;
import net.megavex.scoreboardlibrary.api.team.TeamDisplay;
import net.megavex.scoreboardlibrary.api.team.TeamManager;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.command.CommandSender;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.HandlerList;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.player.PlayerChangedWorldEvent;
import org.bukkit.event.player.PlayerMoveEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.scheduler.BukkitTask;
import org.jetbrains.annotations.NotNull;
import xyz.twovb.sgm.SGM;
import xyz.twovb.sgm.games.GameManager;
import xyz.twovb.sgm.games.Minigame;
import xyz.twovb.sgm.games.impl.capturethebrick.events.PlayerJailedEvent;
import xyz.twovb.sgm.levels.LevelManager;
import xyz.twovb.toolbox.api.CustomPlayer;
import xyz.twovb.toolbox.utils.ChatUtils;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

public class CTB implements Minigame {

    final static String name = "CaptureTheBrick";
    final static FileConfiguration messages = SGM.getInstance().getConfigManager().loadConfig("games/" + name.toLowerCase() + "/messages.yml");
    private final UUID uuid;
    private final ArrayList<Location> redBrickSpawns = new ArrayList<Location>();
    private final ArrayList<Location> blueBrickSpawns = new ArrayList<Location>();
    private final ArrayList<Location> redSpawnArea = new ArrayList<Location>();
    private final ArrayList<Location> blueSpawnArea = new ArrayList<Location>();
    //    private final FileConfiguration messages;
    private final ArrayList<Player> players = new ArrayList<Player>();
    ScoreboardTeam redTeam;
    ScoreboardTeam blueTeam;
    private final HashMap<Player, CtbData.PlayerState> playerStateMap = new HashMap<>();
    private final HashMap<Player, Location> playerLocationHashMap = new HashMap<>();
    private Location redJail;
    private Location blueJail;
    private World gameWorld;
    private GameState state;
    private int bricks;
    private int territoryLevel;
    private int boundsLevel;
    private Location spawnLoc;
    private TeamManager teamManager;
    private Sidebar sb;
    private SidebarAnimation<Component> headerAnimation;
    private ComponentSidebarLayout layout;
    private CommandSender owner;

    private int taskId;

    public CTB() {
        this.uuid = UUID.randomUUID();
        this.state = GameState.PRESTART;
//        this.messages = SGM.getInstance().getConfigManager().loadConfig("games/" + name.toLowerCase() + "/messages.yml");
    }

    public static FileConfiguration getMessages() {
        return messages;
    }

    public void init(CommandSender owner, String world) throws IOException {
        // Perform initialization logic here
        gameWorld = createGameWorld(world, uuid);
        gameWorld.setAutoSave(false);
        if (applyOptions(world)) {
            this.owner = owner;
            this.teamManager = SGM.getInstance().getScoreboardLibrary().createTeamManager();
            this.redTeam = teamManager.createIfAbsent("red_team");
            this.blueTeam = teamManager.createIfAbsent("blue_team");
            Bukkit.getPluginManager().registerEvents(this, SGM.getInstance());

            setupTeamDisplay(redTeam, "Red Team", NamedTextColor.RED);
            setupTeamDisplay(blueTeam, "Blue Team", NamedTextColor.BLUE);

            this.sb = SGM.getInstance().getScoreboardLibrary().createSidebar();
            this.headerAnimation = createGradientAnimation(ChatUtils.translate("<bold>Capture The Brick"));
            var title = SidebarComponent.animatedLine(headerAnimation);

            SimpleDateFormat dtf = new SimpleDateFormat("dd/MM/yy | HH:mm:ss");

            SidebarComponent gameInfo = SidebarComponent.builder()
                    // @formatter:off
                    .addDynamicLine(() -> {
                        var time = dtf.format(new Date());
                        return Component.text(time, NamedTextColor.GRAY);
                    })
                    .addBlankLine()
                    .addDynamicLine(() -> ChatUtils.translate("<red>Red: <bold>" + getBrickCount(redTeam)))
                    .addDynamicLine(() -> ChatUtils.translate("<blue>Blue: <bold>" + getBrickCount(blueTeam)))
                    .addBlankLine()
                    .build();
            // @formatter:on

            this.layout = new ComponentSidebarLayout(title, gameInfo);
            layout.apply(sb);

            // Ready
            state = GameState.READY;
            for (Player player : Bukkit.getOnlinePlayers()) {
                CustomPlayer customPlayer = new CustomPlayer(player);
                customPlayer.sendMessage(messages.getString("system.game.ready").replace("%game%", name));
            }
        } else {
            owner.sendMessage(ChatUtils.translate(messages.getString("system.game.failed").replace("%cause%", "to load options.")));
            this.stop();
        }
    }

    private String getBrickCount(ScoreboardTeam team) {
        String icon = "\uD83D\uDDC3";
        StringBuilder builder = new StringBuilder();
        for (int i = 1; i <= this.bricks; i++) {
            builder.append(icon + " ");
        }
        return builder.toString();
    }

    @Override
    public void onTick() {
        try {
            headerAnimation.nextFrame();
            layout.apply(sb);
        } catch (IllegalStateException ignored) {
        }

        for (Player player : gameWorld.getPlayers()) {
            if (!players.contains(player)) {
//                for (Player gamePlayer : players) {
//                    gamePlayer.hidePlayer(SGM.getInstance(), player);
//                }
                sendMessageToAllPlayers(player.getName() + " IS A IMPOSTER");
            }
        }
        for (Player player : players) {
            CtbData.Territory territory = getTerritory(player);
            player.sendActionBar(ChatUtils.translate(territory.getFancyText()));
        }
    }

    @Override
    public void start(CommandSender sender) {
        if (checkStart(sender)) {
            for (Player player : players) {
                // TODO: REMEMBER TO REMOVE THIS BEFORE USING IT AT CAMP
                player.setAllowFlight(true);
                player.setFlying(true);
                player.teleport(this.spawnLoc);
                sb.addPlayer(player);
                playerStateMap.put(player, CtbData.PlayerState.Free);
            }
            for (int i = 0; i <= this.bricks - 1; i++) {
                placeBrick(redBrickSpawns.get(i), redTeam);
                placeBrick(blueBrickSpawns.get(i), blueTeam);
            }
            BukkitTask task = Bukkit.getScheduler().runTaskTimer(SGM.getInstance(), this::onTick, 0L, 1L);
            this.taskId = task.getTaskId();

            state = GameState.STARTED;
            sendMessageToAllPlayers("The game has started!");
        }
    }

    boolean isJailed(Player player) {
        return playerStateMap.get(player) == CtbData.PlayerState.Jailed;
    }

    CtbData.Territory getTerritory(Player player) {
        Location playerLoc = player.getLocation();
        Location blockCheck = new Location(gameWorld, playerLoc.getBlockX(), territoryLevel, playerLoc.getBlockZ());
        Block territoryCheck = blockCheck.getBlock();

        switch (territoryCheck.getType()) {
            case RED_CONCRETE:
                return CtbData.Territory.Red;
            case BLUE_CONCRETE:
                return CtbData.Territory.Blue;
            case WHITE_CONCRETE:
                return CtbData.Territory.Neutral;
            default:
                return CtbData.Territory.Unknown;
        }
    }

    // Check if we should be starting rn
    private boolean checkStart(CommandSender sender) {
        if (state != GameState.READY) return false;
        for (Player player : players) {
            if (!teamManager.players().contains(player)) {
                sender.sendMessage("All players need to be on a team!");
                return false;
            }
        }
        return true;
    }

    @Override
    public void stop() {
        // TODO: make it wait before unloading world
        // Stop game logic
        state = GameState.ENDING;
        Player winner;
        if (players.size() <= 1) {
            winner = players.get(0);
            winner.sendMessage("Congratulations on winning the game!");
        }
        players.clear(); // Clear player list
        GameManager.getActiveGames().remove(uuid);
        HandlerList.unregisterAll(this);
        for (Player player : gameWorld.getPlayers()) {
            player.teleport(Bukkit.getServer().getWorlds().get(0).getSpawnLocation());
        }
        Bukkit.unloadWorld(gameWorld, false);
        if (teamManager != null) {
            teamManager.close();
        }
        if (sb != null) {
            sb.close();
        }
        Bukkit.getScheduler().cancelTask(this.taskId);
    }


    @Override
    public void addPlayer(Player player) {
        player.teleport(this.spawnLoc);
        players.add(player);
        sendMessageToAllPlayers(messages.getString("system.player.join").replace("%player%", player.getName()));
    }

    @Override
    public void removePlayer(Player player) {
        sendMessageToAllPlayers(messages.getString("system.player.left").replace("%player%", player.getName()));
        sb.removePlayer(player);
        removePlayerFromTeams(player);
        players.remove(player);
    }

    @EventHandler
    void onPlayerQuit(PlayerQuitEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player)) {
            removePlayer(event.getPlayer());
        }
    }

    @EventHandler
    void onWorldChange(PlayerChangedWorldEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player) && event.getFrom().equals(gameWorld)) {
            removePlayer(event.getPlayer());
        }
    }

    @EventHandler
    void onPlayerDeath(PlayerDeathEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player) && state == GameState.STARTED) {
            sendMessageToAllPlayers(player.getName() + " was eliminated!");
            event.setDeathMessage("");
            removePlayer(player);
        }
    }

    @EventHandler
    void onPlayerMove(PlayerMoveEvent event) {
        if (!(event.getFrom().getX() != event.getTo().getX() || event.getFrom().getZ() != event.getTo().getZ())) return;
        Player player = event.getPlayer();
        if (!players.contains(player)) return;
        if (this.state != GameState.STARTED) return;
        if (playerStateMap.get(player) != CtbData.PlayerState.Jailed) return;
        CtbData.Territory territory = getTerritory(player);
        Location jail = gameWorld.getSpawnLocation();
        if (territory.equals(CtbData.Territory.Blue)) {
            jail = this.blueJail;
        } else if (territory.equals(CtbData.Territory.Red)) {
            jail = this.redJail;
        }
        double check = jail.distanceSquared(player.getLocation());
        if (check >= 9) {
            Location lastValidPos = playerLocationHashMap.get(player);
            player.teleport(Objects.requireNonNullElse(lastValidPos, jail));
        } else {
            playerLocationHashMap.put(player, player.getLocation());
        }
    }


    @EventHandler
    void onPlayerJail(PlayerJailedEvent event) {
        CustomPlayer player = new CustomPlayer(event.getJailed());
        player.sendMessage(messages.getString("system.player.jail.info"));
        for (String teamMembers : getTeam(player.player()).defaultDisplay().entries()) {
            Player teamPlayer = Bukkit.getServer().getPlayerExact(teamMembers);
            CustomPlayer teamMember = new CustomPlayer(teamPlayer);
            SGM.getInstance().getCLogger().log(teamMember.player().getName());
            if (teamMember.player() == player.player()) continue;
            teamMember.sendMessage(LegacyComponentSerializer.legacyAmpersand().serialize(event.getMessage()));
        }
    }

    public void addPlayerToJail(Player player) {
        playerStateMap.replace(player, CtbData.PlayerState.Free, CtbData.PlayerState.Jailed);
        PlayerJailedEvent jailedEvent = new PlayerJailedEvent(player);
        jailedEvent.callEvent();
        ScoreboardTeam team = getTeam(player);
        if (team == redTeam) {
            player.teleport(blueJail);
        } else if (team == blueTeam) {
            player.teleport(redJail);
        }
    }

    public void freePlayerFromJail(Player player) {
        playerStateMap.replace(player, CtbData.PlayerState.Jailed, CtbData.PlayerState.Free);
    }

    @Override
    public List<Player> getPlayers() {
        return players;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public UUID getGameId() {
        return uuid;
    }

    @Override
    public GameState getState() {
        return state;
    }

    @Override
    public World getWorld() {
        return gameWorld;
    }

    public void addPlayerToTeam(Player player, CtbData.Teams team) {
        if (teamManager.players().contains(player)) {
            getTeam(player).defaultDisplay().removeEntry(player.getName());
        }
        if (team.equals(CtbData.Teams.Blue)) {
            blueTeam.defaultDisplay().addEntry(player.getName());
        } else if (team.equals(CtbData.Teams.Red)) {
            redTeam.defaultDisplay().addEntry(player.getName());
        }
        this.teamManager.addPlayer(player);
    }

    public void removePlayerFromTeams(Player player) {
        getTeam(player).defaultDisplay().removeEntry(player.getName());
        this.teamManager.removePlayer(player);
    }

    public ScoreboardTeam getTeam(Player player) {
        for (ScoreboardTeam team : teamManager.teams()) {
            if (team.defaultDisplay().entries().contains(player.getName())) {
                return team;
            }
        }
        return null;
    }

    private void placeBrick(Location location, ScoreboardTeam team) {
        Block brick = location.getBlock();
        if (team.equals(redTeam)) {
            brick.setType(Material.RED_WOOL);
        }
        if (team.equals(blueTeam)) {
            brick.setType(Material.BLUE_WOOL);
        }
    }

    private void setupTeamDisplay(ScoreboardTeam team, String displayName, NamedTextColor color) {
        TeamDisplay display = team.defaultDisplay();
        display.displayName(Component.text(displayName));
        display.playerColor(color);
    }

    private @NotNull SidebarAnimation<Component> createGradientAnimation(@NotNull Component text) {
        float step = 1f / 8f;
        TagResolver.Single textPlaceholder = Placeholder.component("text", text);
        List<Component> frames = new ArrayList<>((int) (2f / step));
        float phase = -1f;

        while (phase <= 1) {
            String gradientTag = String.format("<gradient:#9bf8ad:#00ff04:%.2f>", phase);
            Component frame = MiniMessage.miniMessage().deserialize(gradientTag + "<text>", textPlaceholder);
            frames.add(frame);
            phase += step;
        }
        return new CollectionSidebarAnimation<>(frames);
    }

    private @NotNull SidebarAnimation<Component> createScrollingAnimation(@NotNull Component text, int frameDelay) {
        String rawText = PlainTextComponentSerializer.plainText().serialize(text);
        int textLength = rawText.length();
        int frameWidth = Math.min(20, textLength); // Use a maximum frame width of 20 or text length, whichever is smaller.

        // Determine the number of frames needed to scroll the text
        int totalFrames = textLength > frameWidth ? textLength - frameWidth + 1 : 1;
        List<Component> frames = new ArrayList<>(totalFrames * frameDelay);

        for (int i = 0; i < totalFrames; i++) {
            int start = i;
            int end = Math.min(i + frameWidth, textLength);
            String frameText = rawText.substring(start, end);

            // Add padding if the frame is shorter than frameWidth
            if (frameText.length() < frameWidth) {
                frameText = frameText + " ".repeat(frameWidth - frameText.length());
            }

            // Add multiple identical frames to slow down the animation
            Component frameComponent = Component.text(frameText, text.style());
            for (int j = 0; j < frameDelay; j++) {
                frames.add(frameComponent);
            }
        }

        return new CollectionSidebarAnimation<>(frames);
    }

    private Location parseString(String string) {
        String[] coords = string.trim().split(",");
        if (coords.length == 3) {
            return new Location(gameWorld, Double.parseDouble(coords[0]), Double.parseDouble(coords[1]), Double.parseDouble(coords[2]));
        } else {
            return null;
        }
    }

    // Attempts to load the game's options as a YamlConfiguration
    // before processing each section in the same order as the file.
    private boolean applyOptions(String worldName) {
        File configFile = new File(LevelManager.mapPath + worldName + "/sgm.yml");
        SGM.getInstance().getCLogger().log(configFile.getPath());
        SGM.getInstance().getCLogger().log(configFile.getAbsolutePath());
        YamlConfiguration config = YamlConfiguration.loadConfiguration(configFile);

        // Set map information
        ConfigurationSection mapInfoSection = config.getConfigurationSection("mapInfo");
        if (mapInfoSection != null) {
            this.bricks = mapInfoSection.getInt("brickCount");
            this.territoryLevel = mapInfoSection.getInt("territory");
            this.boundsLevel = mapInfoSection.getInt("boundary");
            this.spawnLoc = parseString(mapInfoSection.getString("spawn"));
        } else {
            SGM.getInstance().getCLogger().error("Invalid map info. Please fix before trying again.");
            return false;
        }

        // Process team information

        CtbData.Teams[] teamsList = CtbData.Teams.values();
        for (CtbData.Teams team : teamsList) {
            String teamName = team.name().toLowerCase();
            ConfigurationSection teamConfig = config.getConfigurationSection("teams." + teamName);
            if (teamConfig != null) {
                List<String> brickSpawns = teamConfig.getStringList("brickSpawns");
                List<String> playerSpawnArea = teamConfig.getStringList("playerSpawnArea");

                if (team.equals(CtbData.Teams.Red)) {
                    this.redJail = parseString(teamConfig.getString("jail"));
                } else if (team.equals(CtbData.Teams.Blue)) {
                    this.blueJail = parseString(teamConfig.getString("jail"));
                }

                // Get brick locations
                if (brickSpawns.size() >= this.bricks) {
                    for (String rawBrickLoc : brickSpawns) {
                        Location brickLoc = parseString(rawBrickLoc);
                        if (team.equals(CtbData.Teams.Red)) {
                            this.redBrickSpawns.add(brickLoc);
                        } else if (team.equals(CtbData.Teams.Blue)) {
                            this.blueBrickSpawns.add(brickLoc);
                        }
                    }
                } else {
                    SGM.getInstance().getCLogger().error("Invalid brick spawns. Please fix before trying again.");
                    return false;
                }

                // Hopefully that works!
                if (playerSpawnArea.size() == 2) {
                    for (String rawSpawnLoc : playerSpawnArea) {
                        Location spawnLoc = parseString(rawSpawnLoc);
                        if (team.equals(CtbData.Teams.Red)) {
                            this.redSpawnArea.add(spawnLoc);
                        } else if (team.equals(CtbData.Teams.Blue)) {
                            this.blueSpawnArea.add(spawnLoc);
                        }
                    }
                } else {
                    SGM.getInstance().getCLogger().error("Invalid spawn area. Please fix before trying again.");
                    return false;
                }
            } else {
                SGM.getInstance().getCLogger().error("Invalid teams. Please fix before trying again.");
                return false;
            }
        }
        return true;
}

}